@@ -12,6 +12,7 @@ from io import BytesIO # TODO DELETE
 from lzma import LZMACompressor, LZMADecompressor, PRESET_EXTREME # TODO DELETE
 from os import remove
 from os.path import isdir, isfile
+from shutil import copyfile
 from subprocess import call, check_output, DEVNULL, PIPE, Popen
 from sys import argv
 from tempfile import NamedTemporaryFile
@@ -57,7 +58,7 @@ class ViReaDB:
         self.version = self.cur.execute("SELECT val FROM meta WHERE key='VERSION' LIMIT 1").fetchone()[0]
         self.ref_name = self.cur.execute("SELECT val FROM meta WHERE key='REF_NAME' LIMIT 1").fetchone()[0]
         ref_seq_xz = self.cur.execute("SELECT val FROM meta WHERE key='REF_SEQ_XZ' LIMIT 1").fetchone()[0]
-        self.ref_seq = decompress_seq(ref_seq_xz)
+        self.ref_seq = decompress_str(ref_seq_xz)
         self.ref_len = len(self.ref_seq)
         self.ref_f = NamedTemporaryFile('w', prefix='vireadb', suffix='.fas', buffering=bufsize)
         self.ref_f.write('%s\n%s\n' % (self.ref_name, self.ref_seq)); self.ref_f.flush()
@@ -197,7 +198,7 @@ class ViReaDB:
         Args:
             ``other`` (``vireadb.ViReaDB``): The other database from which to add all entries
 
-            ``check_meta`` (``bool``): Check that the metadata are identical across the two databases
+            ``check_meta`` (``bool``): Check that the metadata are identical across the two databases. Should only be skipped if user is already guaranteed that they match
 
             ``check_unique`` (``bool``): Check that every ID is unique (i.e., no IDs in ``other`` already exist in the calling object). Should only be skipped if user is already guaranteed to not have duplicates
 
@@ -256,7 +257,7 @@ class ViReaDB:
         if tmp is None:
             raise KeyError("ID doesn't exist in database: %s" % ID)
         cram, pos_counts_xz, ins_counts_xz, consensus_xz = tmp
-        return cram, decompress_pos_counts(pos_counts_xz), decompress_ins_counts(ins_counts_xz), decompress_seq(consensus_xz)
+        return cram, decompress_pos_counts(pos_counts_xz), decompress_ins_counts(ins_counts_xz), decompress_str(consensus_xz)
 
     def get_IDs(self):
         '''Return the IDs in this database
@@ -352,8 +353,9 @@ class ViReaDB:
         # decompress counts, compute consensus, and save
         pos_counts = decompress_pos_counts(pos_counts_xz)
         ins_counts = decompress_ins_counts(ins_counts_xz)
-        consensus = compute_consensus(pos_counts, ins_counts, min_depth=min_depth, min_freq=min_freq, ambig=ambig)
-        consensus_xz = compress_seq(consensus)
+        consensus_seq = compute_consensus(pos_counts, ins_counts, min_depth=min_depth, min_freq=min_freq, ambig=ambig)
+        consensus = ">%s (vireadb v%s, min_depth=%s, min_freq=%s, ambig=%s, remove_gaps=%s)\n%s\n" % (ID, self.version, min_depth, min_freq, ambig, remove_gaps, consensus_seq)
+        consensus_xz = compress_str(consensus)
         self.cur.execute("UPDATE seqs SET CONSENSUS_XZ=? WHERE ID=?", (consensus_xz, ID))
         if commit:
             self.commit()
@@ -365,12 +367,29 @@ class ViReaDB:
             ``ID`` (``str``): The unique ID of the entry whose counts to return
 
         Returns:
-            The consensus sequence for ``ID`` (or ``None`` if not yet computed)
+            The consensus sequence for ``ID`` as a FASTA string (or ``None`` if not yet computed)
         '''
         tmp = self.cur.execute("SELECT CONSENSUS_XZ FROM seqs WHERE ID='%s' LIMIT 1" % ID).fetchone()
         if tmp is None:
             raise KeyError("ID doesn't exist in database: %s" % ID)
-        return decompress_seq(tmp[0])
+        return decompress_str(tmp[0])
+
+    def export_cram(self, ID, out_fn, overwrite=False):
+        '''Export the CRAM file of a given entry
+
+        Args:
+            ``ID`` (``str``): The unique ID of the entry whose CRAM to export
+
+            ``out_fn`` (``str``): The path of the output CRAM file
+
+            ``overwrite`` (``bool``): Overwrite output file if it exists
+        '''
+        if isfile(out_fn) and not overwrite:
+            raise ValueError("Output file exists: %s" % out_fn)
+        tmp = self.cur.execute("SELECT CRAM FROM seqs WHERE ID='%s' LIMIT 1" % ID).fetchone()
+        if tmp is None:
+            raise KeyError("ID doesn't exist in database: %s" % ID)
+        f = open(out_fn, 'wb'); f.write(tmp[0]); f.close()
 
     def export_fasta(self, out_fn, IDs, overwrite=False):
         '''Export multiple consensus sequences as a FASTA file
@@ -408,6 +427,8 @@ def create_db(db_fn, ref_fn, overwrite=False, bufsize=DEFAULT_BUFSIZE):
 
         ``overwrite`` (``bool``): Overwrite ``db_fn`` if it already exists
 
+        ``bufsize`` (``int``): Buffer size for reading from file
+
     Returns:
         ``ViReaDB`` object
     '''
@@ -427,7 +448,7 @@ def create_db(db_fn, ref_fn, overwrite=False, bufsize=DEFAULT_BUFSIZE):
 
     # load reference genome
     ref_name, ref_seq = load_ref(ref_fn)
-    ref_seq_xz = compress_seq(ref_seq)
+    ref_seq_xz = compress_str(ref_seq)
 
     # index reference genome
     mmi_f = NamedTemporaryFile('w', prefix='vireadb', suffix='.mmi', buffering=bufsize)
@@ -459,3 +480,39 @@ def load_db(db_fn):
     if not isfile(db_fn):
         raise ValueError("db_fn not found: %s" % db_fn)
     return ViReaDB(db_fn)
+
+def merge_dbs(out_db_fn, in_db_fns, check_meta=True, overwrite=False):
+    '''Merge multiple ViReaDB databases
+
+    Args:
+        ``out_db_fn`` (``str``): The filename of the SQLite3 database file representing the output database
+
+        ``in_db_fns`` (``list``): The filenames of the SQLite3 databases representing the input databases
+
+        ``check_meta`` (``bool``): Check that the metadata are identical across the databases. Should only be skipped if user is already guaranteed that they match
+
+        ``overwrite`` (``bool``): Overwrite ``db_fn`` if it already exists
+
+    Returns:
+        ``ViReaDB`` object
+    '''
+    # validity check
+    for in_db_fn in in_db_fns:
+        if not isfile(in_db_fn):
+            raise ValueError("Input database file not found: %s" % in_db_fn)
+    if isfile(out_db_fn):
+        if overwrite:
+            remove(out_db_fn)
+        else:
+            raise ValueError("out_db_fn exists: %s" % out_db_fn)
+
+    # merge databases
+    copyfile(in_db_fns[0], out_db_fn)
+    out_db = load_db(out_db_fn); out_db_meta = out_db.get_meta()
+    for in_db_fn in in_db_fns[1:]:
+        curr_db = load_db(in_db_fn)
+        if check_meta and out_db_meta != curr_db.get_meta():
+            del out_db; remove(out_db_fn)
+            raise TypeError("Metadata of the databases do not match")
+        out_db.add_all_entries(curr_db)
+    return out_db
